/*
 * This file is part of GtkJson.
 *
 * Copyright © YEAR(S) Canek Peláez Valdés canek@ciencias.unam.mx
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * Namespace for the GtkJson Gtk application.
 */
namespace GtkJson.Application {

    public class NodeRow : GLib.Object {

        public Json.Node node { get; private set; }

        public string node_type { get; private set; }
        public string node_key { get; private set; }
        public string node_value { get; private set; }

        public string[] css_type { get; private set; }
        public string[] css_key { get; private set; }
        public string[] css_value { get; private set; }

        public NodeRow(Json.Node node, string key="") {
            node_key = (quote_key(key)) ? @"“$(key)”" : key;
            switch (node.get_node_type()) {
            case Json.NodeType.ARRAY:
                node_type = "[]";
                css_type = new string[] { "array-type" };
                css_key = new string[] { "array-key" };
                break;
            case Json.NodeType.NULL:
                node_type = "␀";
                css_type = new string[] { "null-type" };
                break;
            case Json.NodeType.OBJECT:
                node_type = "{}";
                css_type = new string[] { "object-type" };
                css_key = new string[] { "object-key" };
                break;
            case Json.NodeType.VALUE:
                node_type = "$";
                node_value = get_value_node_text(node);
                css_type = new string[] { "value-type" };
                css_key = new string[] { "value-key" };
                css_value = new string[] { "value-value" };
                break;
            }
            this.node = node;
        }

        private bool quote_key(string key) {
            if (key == "-" || key == ".")
                return true;
            int d = 0;
            for (int i = 0; i < key.length; i++) {
                char c = key.get(i);
                if (c == '-') {
                    if (i > 0)
                        return true;
                    else
                        continue;
                }
                if (c == '.') {
                    if (d++ > 0)
                        return true;
                    else
                        continue;
                }
                if (c.isdigit())
                    continue;
                return true;
            }
            return false;
        }

        private string get_value_node_text(Json.Node node) {
            var val = node.get_value();
            if (val.holds(typeof(string)))
                return "“" + val.get_string() + "”";
            if (val.holds(typeof(float)))
                return "%g".printf(val.get_float());
            if (val.holds(typeof(double)))
                return "%g".printf(val.get_double());
            if (val.holds(typeof(uchar)))
                return "%c".printf((int)val.get_uchar());
            if (val.holds(typeof(uint)))
                return "%u".printf(val.get_uint());
            if (val.holds(typeof(ulong)))
                return "%lu".printf(val.get_ulong());
            if (val.holds(typeof(uint64)))
                return "%lu".printf((ulong)val.get_uint64());
            if (val.holds(typeof(int)))
                return "%d".printf(val.get_int());
            if (val.holds(typeof(long)))
                return "%ld".printf(val.get_long());
            if (val.holds(typeof(int64)))
                return "%ld".printf((long)val.get_int64());
            if (val.holds(typeof(bool)))
                return val.get_boolean() ? "true" : "false";
            return "<?>";
        }
    }
}
